@@ -16,9 +16,9 @@
 #'   Can be tuned.
 #' @param bias Logical for whether to include bias terms. Can be tuned.
 #' @param epochs An integer for the number of training iterations. Can be tuned.
-#' @param penalty A number for the regularization penalty (lambda). Default `0` 
+#' @param penalty A number for the regularization penalty (lambda). Default `0`
 #'   (no regularization). Higher values increase regularization strength. Can be tuned.
-#' @param mixture A number between 0 and 1 for the elastic net mixing parameter. 
+#' @param mixture A number between 0 and 1 for the elastic net mixing parameter.
 #'   Default `0` (pure L2/Ridge regularization).
 #'   - `0`: Pure L2 regularization (Ridge)
 #'   - `1`: Pure L1 regularization (Lasso)
@@ -28,16 +28,25 @@
 #' @param learn_rate A number for the learning rate. Can be tuned.
 #' @param optimizer A character string for the optimizer type ("adam", "sgd",
 #'   "rmsprop"). Can be tuned.
-#' @param optimizer_args A named list of additional arguments passed to the optimizer.
-#'   Cannot be tuned.
-#' @param loss A character string for the loss function ("mse", "mae",
-#'   "cross_entropy", "bce"). Cannot be tuned.
 #' @param validation_split A number between 0 and 1 for the proportion of data
 #'   used for validation. Can be tuned.
-#' @param device A character string for the device to use ("cpu", "cuda", "mps").
-#'   If NULL, auto-detects available GPU. Cannot be tuned.
-#' @param verbose Logical for whether to print training progress. Default FALSE.
-#'   Cannot be tuned.
+#'
+#' @param optimizer_args A named list of additional arguments passed to the
+#'   optimizer. Cannot be tuned — pass via `set_engine()`.
+#' @param loss A character string for the loss function ("mse", "mae",
+#'   "cross_entropy", "bce"). Cannot be tuned — pass via `set_engine()`.
+#' @param architecture An [nn_arch()] object for a custom architecture. Cannot
+#'   be tuned — pass via `set_engine()`.
+#' @param flatten_input Logical or `NULL`. Controls input flattening. Cannot be
+#'   tuned — pass via `set_engine()`.
+#' @param early_stopping An [early_stop()] object or `NULL`. Cannot be tuned —
+#'   pass via `set_engine()`.
+#' @param device A character string for the device ("cpu", "cuda", "mps").
+#'   Cannot be tuned — pass via `set_engine()`.
+#' @param verbose Logical for whether to print training progress. Cannot be
+#'   tuned — pass via `set_engine()`.
+#' @param cache_weights Logical. If `TRUE`, stores trained weight matrices in
+#'   the returned object. Cannot be tuned — pass via `set_engine()`.
 #'
 #' @details
 #' This function creates a model specification for a feedforward neural network
@@ -49,19 +58,9 @@
 #' - Hyperparameter tuning integration
 #' - Both regression and classification tasks
 #'
-#' The `hidden_neurons` parameter accepts an integer vector where each element
-#' represents the number of neurons in that hidden layer. For example,
-#' `hidden_neurons = c(128, 64, 32)` creates a network with three hidden layers.
-#'
-#' The `device` parameter controls where computation occurs:
-#' - `NULL` (default): Auto-detect best available device (CUDA > MPS > CPU)
-#' - `"cuda"`: Use NVIDIA GPU
-#' - `"mps"`: Use Apple Silicon GPU
-#' - `"cpu"`: Use CPU only
-#'
-#' When tuning, you can use special tune tokens:
-#' - For `hidden_neurons`: use `tune("hidden_neurons")` with a custom range
-#' - For `activation`: use `tune("activation")` with values like "relu", "tanh"
+#' Parameters that cannot be tuned (`architecture`, `flatten_input`,
+#' `early_stopping`, `device`, `verbose`, `cache_weights`, `optimizer_args`,
+#' `loss`) must be set via `set_engine()`, not as arguments to `mlp_kindling()`.
 #'
 #' @return A model specification object with class `mlp_kindling`.
 #'
@@ -75,6 +74,11 @@
 #'         parsnip[fit]
 #'     )
 #'
+#'     # library(recipes)
+#'     # library(workflows)
+#'     # library(parsnip)
+#'     # library(tune)
+#'
 #'     # Model specs
 #'     mlp_spec = mlp_kindling(
 #'         mode = "classification",
@@ -102,10 +106,11 @@
 #' }
 #'
 #' @export
-mlp_kindling =
+mlp_kindling = 
     function(
         mode = "unknown",
         engine = "kindling",
+        # ---- Tunable parameters ----
         hidden_neurons = NULL,
         activations = NULL,
         output_activation = NULL,
@@ -116,46 +121,62 @@ mlp_kindling =
         mixture = NULL,
         learn_rate = NULL,
         optimizer = NULL,
+        validation_split = NULL,
+        # ---- Non-tunable parameters (engine-level, set via set_engine()) ----
         optimizer_args = NULL,
         loss = NULL,
-        validation_split = NULL,
+        architecture = NULL,
+        flatten_input = NULL,
+        early_stopping = NULL,
         device = NULL,
-        verbose = NULL
-    ) {
-        
-        if (!requireNamespace("parsnip", quietly = TRUE)) {
-            cli::cli_abort("Package {.pkg parsnip} is required but not installed.")
-        }
-        
-        args = list(
-            hidden_neurons = rlang::enquo(hidden_neurons),
-            activations = rlang::enquo(activations),
-            output_activation = rlang::enquo(output_activation),
-            bias = rlang::enquo(bias),
-            epochs = rlang::enquo(epochs),
-            batch_size = rlang::enquo(batch_size),
-            penalty = rlang::enquo(penalty),
-            mixture = rlang::enquo(mixture),
-            learn_rate = rlang::enquo(learn_rate),
-            optimizer = rlang::enquo(optimizer),
-            optimizer_args = rlang::enquo(optimizer_args),
-            loss = rlang::enquo(loss),
-            validation_split = rlang::enquo(validation_split),
-            device = rlang::enquo(device),
-            verbose = rlang::enquo(verbose)
-        )
-        
-        parsnip::new_model_spec(
-            "mlp_kindling",
-            args = args,
-            eng_args = NULL,
-            mode = mode,
-            user_specified_mode = !missing(mode),
-            method = NULL,
-            engine = engine,
-            user_specified_engine = !missing(engine)
-        )
+        verbose = NULL,
+        cache_weights = NULL
+    )
+{
+    
+    if (!requireNamespace("parsnip", quietly = TRUE)) {
+        cli::cli_abort("Package {.pkg parsnip} is required but not installed.")
     }
+    
+    # ---- "Tunable" args ----
+    args = list(
+        hidden_neurons = rlang::enquo(hidden_neurons),
+        activations = rlang::enquo(activations),
+        output_activation = rlang::enquo(output_activation),
+        bias = rlang::enquo(bias),
+        epochs = rlang::enquo(epochs),
+        batch_size = rlang::enquo(batch_size),
+        penalty = rlang::enquo(penalty),
+        mixture = rlang::enquo(mixture),
+        learn_rate = rlang::enquo(learn_rate),
+        optimizer = rlang::enquo(optimizer),
+        validation_split = rlang::enquo(validation_split)
+    )
+    
+    # ---- "Non-tunable" engine args ----
+    eng_args = list(
+        optimizer_args = rlang::enquo(optimizer_args),
+        loss = rlang::enquo(loss),
+        architecture = rlang::enquo(architecture),
+        flatten_input = rlang::enquo(flatten_input),
+        early_stopping = rlang::enquo(early_stopping),
+        device = rlang::enquo(device),
+        verbose = rlang::enquo(verbose),
+        cache_weights = rlang::enquo(cache_weights)
+    )
+    
+    parsnip::new_model_spec(
+        "mlp_kindling",
+        args = args,
+        eng_args = eng_args,
+        mode = mode,
+        user_specified_mode = !missing(mode),
+        method = NULL,
+        engine = engine,
+        user_specified_engine = !missing(engine)
+    )
+}
+
 
 #' @export
 print.mlp_kindling = function(x, ...) {
@@ -164,9 +185,10 @@ print.mlp_kindling = function(x, ...) {
     invisible(x)
 }
 
+
 #' @export
 #' @importFrom stats update
-update.mlp_kindling =
+update.mlp_kindling = 
     function(
         object,
         parameters = NULL,
@@ -180,42 +202,44 @@ update.mlp_kindling =
         mixture = NULL,
         learn_rate = NULL,
         optimizer = NULL,
+        validation_split = NULL,
         optimizer_args = NULL,
         loss = NULL,
-        validation_split = NULL,
+        architecture = NULL,
+        flatten_input = NULL,
+        early_stopping = NULL,
         device = NULL,
         verbose = NULL,
+        cache_weights = NULL,
         fresh = FALSE,
         ...
-    ) {
-        
-        args = list(
-            hidden_neurons = rlang::enquo(hidden_neurons),
-            activations = rlang::enquo(activations),
-            output_activation = rlang::enquo(output_activation),
-            bias = rlang::enquo(bias),
-            epochs = rlang::enquo(epochs),
-            batch_size = rlang::enquo(batch_size),
-            penalty = rlang::enquo(penalty),
-            mixture = rlang::enquo(mixture),
-            learn_rate = rlang::enquo(learn_rate),
-            optimizer = rlang::enquo(optimizer),
-            optimizer_args = rlang::enquo(optimizer_args),
-            loss = rlang::enquo(loss),
-            validation_split = rlang::enquo(validation_split),
-            device = rlang::enquo(device),
-            verbose = rlang::enquo(verbose)
-        )
-        
-        parsnip::update_spec(
-            object = object,
-            parameters = parameters,
-            args_enquo_list = args,
-            fresh = fresh,
-            cls = "mlp_kindling",
-            ...
-        )
-    }
+    )
+{
+    
+    args = list(
+        hidden_neurons = rlang::enquo(hidden_neurons),
+        activations = rlang::enquo(activations),
+        output_activation = rlang::enquo(output_activation),
+        bias = rlang::enquo(bias),
+        epochs = rlang::enquo(epochs),
+        batch_size = rlang::enquo(batch_size),
+        penalty = rlang::enquo(penalty),
+        mixture = rlang::enquo(mixture),
+        learn_rate = rlang::enquo(learn_rate),
+        optimizer = rlang::enquo(optimizer),
+        validation_split = rlang::enquo(validation_split)
+    )
+    
+    parsnip::update_spec(
+        object = object,
+        parameters = parameters,
+        args_enquo_list = args,
+        fresh = fresh,
+        cls = "mlp_kindling",
+        ...
+    )
+}
+
 
 #' @export
 #' @importFrom parsnip translate
@@ -223,11 +247,11 @@ translate.mlp_kindling = function(x, engine = x$engine, ...) {
     if (is.null(engine)) {
         cli::cli_abort("Please set an engine with `set_engine()`.")
     }
-    
     x = parsnip::translate.default(x, engine, ...)
     x
 }
 
+
 #' @export
 #' @importFrom tune tunable
 tunable.mlp_kindling = function(x, ...) {
@@ -255,3 +279,36 @@ tunable.mlp_kindling = function(x, ...) {
         component_id = "main"
     )
 }
+
+#' Basemodels-tidymodels wrappers
+#'
+#' @param formula A formula specifying the model (e.g., `y ~ x1 + x2`)
+#' @param data A data frame containing the training data
+#' @param ... Additional arguments passed to the underlying training function
+#'
+#' @return
+#' * `ffnn_wrapper()` returns an object of class `"ffnn_fit"` containing the trained feedforward neural network model and metadata. See [ffnn()] for details.
+#' * `rnn_wrapper()` returns an object of class `"rnn_fit"` containing the trained recurrent neural network model and metadata. See [rnn()] for details.
+#'
+#' @details
+#' These wrapper functions are designed to interface with the `{tidymodels}` 
+#' ecosystem, particularly for use with [tune::tune_grid()] and workflows.
+#' They handle the conversion of tuning parameters (especially list-column 
+#' parameters from [grid_depth()]) into the format expected by [ffnn()] and [rnn()].
+#' 
+#' @rdname kindling-nn-wrappers
+#' @section FFNN (MLP) Wrapper for `{tidymodels}` interface:
+#' This is a function to interface into `{tidymodels}`
+#' (do not use this, use `kindling::ffnn()` instead).
+#' 
+#' @keywords internal
+#' @export
+ffnn_wrapper = function(formula, data, ...) {
+    dots = list(...)
+    dots = prepare_kindling_args(dots)
+    
+    do.call(
+        ffnn,
+        c(list(formula = formula, data = data), dots)
+    )
+}
