@@ -6,7 +6,7 @@
 #'
 #' @inheritParams mlp_kindling
 #' @param rnn_type A character string for the type of RNN cell ("rnn", "lstm",
-#'   "gru"). Cannot be tuned.
+#'   "gru"). Cannot be tuned — pass via `set_engine()`.
 #' @param bidirectional A logical indicating whether to use bidirectional RNN.
 #'   Can be tuned.
 #' @param dropout A number between 0 and 1 for dropout rate between layers.
@@ -62,12 +62,12 @@
 #' }
 #'
 #' @export
-rnn_kindling =
+rnn_kindling = 
     function(
         mode = "unknown",
         engine = "kindling",
+        # ---- Tunable parameters ----
         hidden_neurons = NULL,
-        rnn_type = NULL,
         activations = NULL,
         output_activation = NULL,
         bias = NULL,
@@ -79,49 +79,61 @@ rnn_kindling =
         mixture = NULL,
         learn_rate = NULL,
         optimizer = NULL,
+        validation_split = NULL,
+        # ---- Non-tunable parameters (engine-level, set via set_engine()) ----
+        rnn_type = NULL,
         optimizer_args = NULL,
         loss = NULL,
-        validation_split = NULL,
+        early_stopping = NULL,
         device = NULL,
-        verbose = NULL
-    ) {
-        
-        if (!requireNamespace("parsnip", quietly = TRUE)) {
-            cli::cli_abort("Package {.pkg parsnip} is required but not installed.")
-        }
-        
-        args = list(
-            hidden_neurons = rlang::enquo(hidden_neurons),
-            rnn_type = rlang::enquo(rnn_type),
-            activations = rlang::enquo(activations),
-            output_activation = rlang::enquo(output_activation),
-            bias = rlang::enquo(bias),
-            bidirectional = rlang::enquo(bidirectional),
-            dropout = rlang::enquo(dropout),
-            epochs = rlang::enquo(epochs),
-            batch_size = rlang::enquo(batch_size),
-            penalty = rlang::enquo(penalty),
-            mixture = rlang::enquo(mixture),
-            learn_rate = rlang::enquo(learn_rate),
-            optimizer = rlang::enquo(optimizer),
-            optimizer_args = rlang::enquo(optimizer_args),
-            loss = rlang::enquo(loss),
-            validation_split = rlang::enquo(validation_split),
-            device = rlang::enquo(device),
-            verbose = rlang::enquo(verbose)
-        )
-        
-        parsnip::new_model_spec(
-            "rnn_kindling",
-            args = args,
-            eng_args = NULL,
-            mode = mode,
-            user_specified_mode = !missing(mode),
-            method = NULL,
-            engine = engine,
-            user_specified_engine = !missing(engine)
-        )
+        verbose = NULL,
+        cache_weights = NULL
+    )
+{
+    
+    if (!requireNamespace("parsnip", quietly = TRUE)) {
+        cli::cli_abort("Package {.pkg parsnip} is required but not installed.")
     }
+    
+    # ---- "Tunable" args ----
+    args = list(
+        hidden_neurons = rlang::enquo(hidden_neurons),
+        activations = rlang::enquo(activations),
+        output_activation = rlang::enquo(output_activation),
+        bias = rlang::enquo(bias),
+        bidirectional = rlang::enquo(bidirectional),
+        dropout = rlang::enquo(dropout),
+        epochs = rlang::enquo(epochs),
+        batch_size = rlang::enquo(batch_size),
+        penalty = rlang::enquo(penalty),
+        mixture = rlang::enquo(mixture),
+        learn_rate = rlang::enquo(learn_rate),
+        optimizer = rlang::enquo(optimizer),
+        validation_split = rlang::enquo(validation_split)
+    )
+    
+    # ---- "Non-tunable" engine args ----
+    eng_args = list(
+        rnn_type = rlang::enquo(rnn_type),
+        optimizer_args = rlang::enquo(optimizer_args),
+        loss = rlang::enquo(loss),
+        early_stopping = rlang::enquo(early_stopping),
+        device = rlang::enquo(device),
+        verbose = rlang::enquo(verbose),
+        cache_weights = rlang::enquo(cache_weights)
+    )
+    
+    parsnip::new_model_spec(
+        "rnn_kindling",
+        args = args,
+        eng_args = eng_args,
+        mode = mode,
+        user_specified_mode = !missing(mode),
+        method = NULL,
+        engine = engine,
+        user_specified_engine = !missing(engine)
+    )
+}
 
 #' @export
 print.rnn_kindling = function(x, ...) {
@@ -132,12 +144,11 @@ print.rnn_kindling = function(x, ...) {
 
 #' @export
 #' @importFrom stats update
-update.rnn_kindling =
+update.rnn_kindling = 
     function(
         object,
         parameters = NULL,
         hidden_neurons = NULL,
-        rnn_type = NULL,
         activations = NULL,
         output_activation = NULL,
         bias = NULL,
@@ -149,45 +160,44 @@ update.rnn_kindling =
         mixture = NULL,
         learn_rate = NULL,
         optimizer = NULL,
+        validation_split = NULL,
+        rnn_type = NULL,
         optimizer_args = NULL,
         loss = NULL,
-        validation_split = NULL,
+        early_stopping = NULL,
         device = NULL,
         verbose = NULL,
+        cache_weights = NULL,
         fresh = FALSE,
         ...
-    ) {
-        
-        args = list(
-            hidden_neurons = rlang::enquo(hidden_neurons),
-            rnn_type = rlang::enquo(rnn_type),
-            activations = rlang::enquo(activations),
-            output_activation = rlang::enquo(output_activation),
-            bias = rlang::enquo(bias),
-            bidirectional = rlang::enquo(bidirectional),
-            dropout = rlang::enquo(dropout),
-            epochs = rlang::enquo(epochs),
-            batch_size = rlang::enquo(batch_size),
-            penalty = rlang::enquo(penalty),
-            mixture = rlang::enquo(mixture),
-            learn_rate = rlang::enquo(learn_rate),
-            optimizer = rlang::enquo(optimizer),
-            optimizer_args = rlang::enquo(optimizer_args),
-            loss = rlang::enquo(loss),
-            validation_split = rlang::enquo(validation_split),
-            device = rlang::enquo(device),
-            verbose = rlang::enquo(verbose)
-        )
-        
-        parsnip::update_spec(
-            object = object,
-            parameters = parameters,
-            args_enquo_list = args,
-            fresh = fresh,
-            cls = "rnn_kindling",
-            ...
-        )
-    }
+    )
+{
+    
+    args = list(
+        hidden_neurons = rlang::enquo(hidden_neurons),
+        activations = rlang::enquo(activations),
+        output_activation = rlang::enquo(output_activation),
+        bias = rlang::enquo(bias),
+        bidirectional = rlang::enquo(bidirectional),
+        dropout = rlang::enquo(dropout),
+        epochs = rlang::enquo(epochs),
+        batch_size = rlang::enquo(batch_size),
+        penalty = rlang::enquo(penalty),
+        mixture = rlang::enquo(mixture),
+        learn_rate = rlang::enquo(learn_rate),
+        optimizer = rlang::enquo(optimizer),
+        validation_split = rlang::enquo(validation_split)
+    )
+    
+    parsnip::update_spec(
+        object = object,
+        parameters = parameters,
+        args_enquo_list = args,
+        fresh = fresh,
+        cls = "rnn_kindling",
+        ...
+    )
+}
 
 #' @export
 #' @importFrom parsnip translate
@@ -207,7 +217,7 @@ tunable.rnn_kindling = function(x, ...) {
         name = c(
             "hidden_neurons", "activations", "output_activation",
             "bias", "bidirectional", "dropout", "epochs", "batch_size",
-            "penalty", "mixture", "learn_rate", "optimizer", 
+            "penalty", "mixture", "learn_rate", "optimizer",
             "validation_split"
         ),
         call_info = list(
@@ -230,3 +240,19 @@ tunable.rnn_kindling = function(x, ...) {
         component_id = "main"
     )
 }
+
+#' @rdname kindling-nn-wrappers
+#' @section RNN Wrapper for `{tidymodels}` interface:
+#' Internal wrapper — use `rnn_kindling()` + `fit()` instead.
+#'
+#' @keywords internal
+#' @export
+rnn_wrapper = function(formula, data, ...) {
+    dots = list(...)
+    dots = prepare_kindling_args(dots)
+    
+    do.call(
+        rnn,
+        c(list(formula = formula, data = data), dots)
+    )
+}
