@@ -1,6 +1,7 @@
 #' @importFrom stats reorder setNames
 #' @importFrom rlang :=
+#' @importFrom lifecycle badge
 NULL
 
 # To avoid R CMD check notes about undefined global variables
-utils::globalVariables(c("x_names", "rel_imp", "object", "new_data"))
+utils::globalVariables(c("x_names", "rel_imp", "object", "new_data", "nn_module", "fit_class"))
